import pyscripts

from discount_api import DiscountService
from mbcontextmessagehandler import MbContextMessageBus


def get_discount_service():
    # type: (...) -> DiscountService

    return DiscountService(
        message_bus=MbContextMessageBus(mbcontext=pyscripts.mbcontext),
    )
