from typing import (
    Dict,
    Optional,
)

from application.service import LoaderUpdaterService


class LoaderUpdaterInteractor(object):
    def __init__(
        self,                       # type: LoaderUpdaterInteractor
        loader_updater_service,     # type: LoaderUpdaterService
    ):
        # type: (...) -> None

        self.loader_updater_service = loader_updater_service

    def update_loaders(
        self,       # type: LoaderUpdaterInteractor
        data=None,  # type: Optional[PosUpdaterDto]
    ):
        # type: (...) -> None

        return self.loader_updater_service.update_loaders(
            restart_hv=True,
            data=data,
        )
