class TableServiceException(Exception):

    def __init__(
        self,           # type: TableServiceException
        description,    # type: str
    ):
        # type: (...) -> None

        super(TableServiceException, self).__init__()

        self._description = description

    @property
    def error_description(
        self
    ):
        # type: (...) -> str

        return self._description

    def __str__(
        self
    ):
        # type: (...) -> str

        return "Error description: {0}".format(self._description)
