class ServiceChild(object):

    def __init__(
        self,           # type: ServiceChild
        service_id,     # type: int
        table_id,       # type: str
        status,         # type: int
        table_name,     # type: str
    ):
        # type: (...) -> None

        self.service_id = service_id
        self.table_id = table_id
        self.status = status
        self.table_name = table_name
