class ReportNotImplemented(Exception):

    def __init__(
        self,
        report_name: str,
    ) -> None:
        super(ReportNotImplemented, self).__init__(f"Report requested was not implemented: {report_name}")
