class ServiceTender(object):
    def __init__(
        self,               # type: ServiceTender
        description,        # type: str
        amount,             # type: float
    ):
        # type: (...) -> None

        self.description = description
        self.amount = amount
