from api.discount import get_discount_service
from typing import List

global_discount_justifications = None


def get_global_discount_justifications():
    # type: (...) -> List[str]

    global global_discount_justifications

    if global_discount_justifications is None:
        global_discount_justifications = get_discount_service().get_justification_config().justifications

    return global_discount_justifications
