class MaxSaleItemsQtyException(Exception):

    def __init__(
        self,    # type: MaxSaleItemsQtyException
        qty,     # type: int
    ):
        # type: (...) -> None

        super(MaxSaleItemsQtyException, self).__init__()

        self.message = "$MAX_ITEMS_TO_FISCALIZE|{}".format(qty)
