from messagebus import TokenCreator, TokenPriority

POS_CONTROL_GROUP = "5"

TK_POS_GET_MODEL = TokenCreator.create_token(
    priority=TokenPriority.high,
    message_group=POS_CONTROL_GROUP,
    number="1"
)

TK_POS_GETCONFIG = TokenCreator.create_token(
    priority=TokenPriority.high,
    message_group=POS_CONTROL_GROUP,
    number="16",
)
