class ProductBarcodeNotFound(Exception):

    def __init__(
        self,       # type: ProductBarcodeNotFound
        part_code,  # type: str
    ):
        # type: (...) -> None
        super(ProductBarcodeNotFound, self).__init__(part_code)

        self.part_code = part_code
