class User(object):
    def __init__(
        self,       # type: User
        id,         # type: int
        name,       # type: str
        level,      # typer: int
    ):
        self.id = id
        self.name = name
        self.level = level
