class BOHOrderPictureRequest(object):
    def __init__(
        self,
        pos_id,    # type: str
        order_id,  # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id
        self.order_id = order_id
