from datetime import datetime

from ._TagEventType import TagEventType


class TagEvent(object):
    def __init__(self, tag, action, date, hallow=False):
        # type: (str, TagEventType, datetime, bool) -> None
        self.tag = tag
        self.action = action
        self.date = date
        self.hallow = hallow
