import pyscripts

from boss_api import Boss
from mbcontextmessagehandler import MbContextMessageBus


def get_boss_service():
    # type: (...) -> Boss

    message_bus = MbContextMessageBus(mbcontext=pyscripts.mbcontext)
    return Boss(message_bus=message_bus)
