class CustomProperty(object):

    def __init__(
        self,
        key=None,      # type: str
        value=None     # type: str
    ):
        # type: (...) -> None

        self.key = key
        self.value = value
